@@ -16,39 +16,38 @@ Assembly
     To assemble code, simply invoke :func:`asm` on the code to assemble.
 
         >>> asm('mov eax, 0')
-        '\xb8\x00\x00\x00\x00'
+        b'\xb8\x00\x00\x00\x00'
 
     Additionally, you can use constants as defined in the :mod:`pwnlib.constants`
     module.
 
         >>> asm('mov eax, SYS_execve')
-        '\xb8\x0b\x00\x00\x00'
+        b'\xb8\x0b\x00\x00\x00'
 
     Finally, :func:`asm` is used to assemble shellcode provided by ``pwntools``
     in the :mod:`shellcraft` module.
 
         >>> asm(shellcraft.nop())
-        '\x90'
+        b'\x90'
 
 Disassembly
 ------------------------
 
     To disassemble code, simply invoke :func:`disasm` on the bytes to disassemble.
 
-    >>> disasm('\xb8\x0b\x00\x00\x00')
-    '   0:   b8 0b 00 00 00          mov    eax,0xb'
+    >>> disasm(b'\xb8\x0b\x00\x00\x00')
+    '   0:   b8 0b 00 00 00          mov    eax, 0xb'
 
 """
 from __future__ import absolute_import
+from __future__ import division
 
 import errno
 import os
 import platform
 import re
 import shutil
-import string
 import subprocess
-import sys
 import tempfile
 from collections import defaultdict
 from glob import glob
@@ -90,7 +89,7 @@ def dpkg_search_for_binutils(arch, util):
 
     try:
         filename = 'bin/%s*linux*-%s' % (arch, util)
-        output = subprocess.check_output(['dpkg','-S',filename])
+        output = subprocess.check_output(['dpkg','-S',filename], universal_newlines = True)
         for line in output.strip().splitlines():
             package, path = line.split(':', 1)
             packages.append(package)
@@ -160,7 +159,6 @@ def which_binutils(util):
         Exception: Could not find 'as' installed for ContextType(arch = 'msp430')
     """
     arch = context.arch
-    bits = context.bits
 
     # Fix up pwntools vs Debian triplet naming, and account
     # for 'thumb' being its own pwntools architecture.
@@ -355,7 +353,8 @@ def _run(cmd, stdin = None):
             cmd,
             stdin  = subprocess.PIPE,
             stdout = subprocess.PIPE,
-            stderr = subprocess.PIPE
+            stderr = subprocess.PIPE,
+            universal_newlines = True
         )
         stdout, stderr = proc.communicate(stdin)
         exitcode = proc.wait()
@@ -400,8 +399,6 @@ def cpp(shellcode):
         >>> cpp("SYS_setresuid", os = "freebsd")
         '311\n'
     """
-    arch = context.arch
-    os   = context.os
     code = _include_header() + shellcode
     cmd  = [
         'cpp',
@@ -465,7 +462,7 @@ def make_elf_from_assembly(assembly,
         >>> sc += shellcraft.echo('Hello\n')
         >>> sc += 'mov rsp, rbp; pop rbp; ret'
         >>> solib = make_elf_from_assembly(sc, shared=1)
-        >>> subprocess.check_output(['echo', 'World'], env={'LD_PRELOAD': solib})
+        >>> subprocess.check_output(['echo', 'World'], env={'LD_PRELOAD': solib}, universal_newlines = True)
         'Hello\nWorld\n'
 
         The same thing can be done with :func:`.make_elf`, though the sizes
@@ -498,7 +495,7 @@ def make_elf_from_assembly(assembly,
     result = asm(assembly, vma = vma, shared = shared, extract = False, **kwargs)
 
     if not extract:
-        os.chmod(result, 0755)
+        os.chmod(result, 0o755)
     else:
         with open(result, 'rb') as io:
             result = io.read()
@@ -531,12 +528,12 @@ def make_elf(data,
         execve('/bin/sh',...).
 
         >>> context.clear(arch='i386')
-        >>> bin_sh = '6a68682f2f2f73682f62696e89e331c96a0b5899cd80'.decode('hex')
+        >>> bin_sh = unhex('6a68682f2f2f73682f62696e89e331c96a0b5899cd80')
         >>> filename = make_elf(bin_sh, extract=False)
         >>> p = process(filename)
-        >>> p.sendline('echo Hello; exit')
+        >>> p.sendline(b'echo Hello; exit')
         >>> p.recvline()
-        'Hello\n'
+        b'Hello\n'
     """
     retval = None
 
@@ -553,7 +550,7 @@ def make_elf(data,
     assembler = _assembler()
     linker    = _linker()
     code      = _arch_header()
-    code      += '.string "%s"' % ''.join('\\x%02x' % ord(c) for c in data)
+    code      += '.string "%s"' % ''.join('\\x%02x' % c for c in bytearray(data))
     code      += '\n'
 
     log.debug("Building ELF:\n" + code)
@@ -564,13 +561,13 @@ def make_elf(data,
     step3     = path.join(tmpdir, 'step3-elf')
 
     try:
-        with open(step1, 'wb+') as f:
+        with open(step1, 'w') as f:
             f.write(code)
 
         _run(assembler + ['-o', step2, step1])
 
         linker_options = ['-z', 'execstack']
-        if vma:
+        if vma is not None:
             linker_options += ['--section-start=.shellcode=%#x' % vma,
                                '--entry=%#x' % vma]
         elif shared:
@@ -585,11 +582,11 @@ def make_elf(data,
             _run([which_binutils('strip'), '--strip-unneeded', step3])
 
         if not extract:
-            os.chmod(step3, 0755)
+            os.chmod(step3, 0o755)
             retval = step3
 
         else:
-            with open(step3, 'r') as f:
+            with open(step3, 'rb') as f:
                 retval = f.read()
     except Exception:
         log.exception("An error occurred while building an ELF:\n%s" % code)
@@ -605,7 +602,7 @@ def asm(shellcode, vma = 0, extract = True, shared = False):
     Runs :func:`cpp` over a given shellcode and then assembles it into bytes.
 
     To see which architectures or operating systems are supported,
-    look in :mod:`pwnlib.contex`.
+    look in :mod:`pwnlib.context`.
 
     Assembling shellcode requires that the GNU assembler is installed
     for the target architecture.
@@ -624,13 +621,13 @@ def asm(shellcode, vma = 0, extract = True, shared = False):
     Examples:
 
         >>> asm("mov eax, SYS_select", arch = 'i386', os = 'freebsd')
-        '\xb8]\x00\x00\x00'
+        b'\xb8]\x00\x00\x00'
         >>> asm("mov eax, SYS_select", arch = 'amd64', os = 'linux')
-        '\xb8\x17\x00\x00\x00'
+        b'\xb8\x17\x00\x00\x00'
         >>> asm("mov rax, SYS_select", arch = 'amd64', os = 'linux')
-        'H\xc7\xc0\x17\x00\x00\x00'
+        b'H\xc7\xc0\x17\x00\x00\x00'
         >>> asm("mov r0, #SYS_select", arch = 'arm', os = 'linux', bits=32)
-        'R\x00\xa0\xe3'
+        b'R\x00\xa0\xe3'
     """
     result = ''
 
@@ -678,10 +675,11 @@ def asm(shellcode, vma = 0, extract = True, shared = False):
 
             _run(linker + ldflags)
 
-        elif file(step2,'rb').read(4) == '\x7fELF':
+        elif open(step2,'rb').read(4) == b'\x7fELF':
             # Sanity check for seeing if the output has relocations
             relocs = subprocess.check_output(
-                [which_binutils('readelf'), '-r', step2]
+                [which_binutils('readelf'), '-r', step2],
+                universal_newlines = True
             ).strip()
             if extract and len(relocs.split('\n')) > 1:
                 log.error('Shellcode contains relocations:\n%s' % relocs)
@@ -693,7 +691,7 @@ def asm(shellcode, vma = 0, extract = True, shared = False):
 
         _run(objcopy + [step3, step4])
 
-        with open(step4) as fd:
+        with open(step4, 'rb') as fd:
             result = fd.read()
 
     except Exception:
@@ -713,9 +711,6 @@ def disasm(data, vma = 0, byte = True, offset = True, instructions = True):
     To see which architectures are supported,
     look in :mod:`pwnlib.contex`.
 
-    To support all these architecture, we bundle the GNU objcopy
-    and objdump with pwntools.
-
     Arguments:
       data(str): Bytestring to disassemble.
       vma(int): Passed through to the --adjust-vma argument of objdump
@@ -727,26 +722,28 @@ def disasm(data, vma = 0, byte = True, offset = True, instructions = True):
 
     Examples:
 
-        >>> print disasm('b85d000000'.decode('hex'), arch = 'i386')
-           0:   b8 5d 00 00 00          mov    eax,0x5d
-        >>> print disasm('b85d000000'.decode('hex'), arch = 'i386', byte = 0)
-           0:   mov    eax,0x5d
-        >>> print disasm('b85d000000'.decode('hex'), arch = 'i386', byte = 0, offset = 0)
-        mov    eax,0x5d
-        >>> print disasm('b817000000'.decode('hex'), arch = 'amd64')
-           0:   b8 17 00 00 00          mov    eax,0x17
-        >>> print disasm('48c7c017000000'.decode('hex'), arch = 'amd64')
-           0:   48 c7 c0 17 00 00 00    mov    rax,0x17
-        >>> print disasm('04001fe552009000'.decode('hex'), arch = 'arm')
+        >>> print(disasm(unhex('b85d000000'), arch = 'i386'))
+           0:   b8 5d 00 00 00          mov    eax, 0x5d
+        >>> print(disasm(unhex('b85d000000'), arch = 'i386', byte = 0))
+           0:   mov    eax, 0x5d
+        >>> print(disasm(unhex('b85d000000'), arch = 'i386', byte = 0, offset = 0))
+        mov    eax, 0x5d
+        >>> print(disasm(unhex('b817000000'), arch = 'amd64'))
+           0:   b8 17 00 00 00          mov    eax, 0x17
+        >>> print(disasm(unhex('48c7c017000000'), arch = 'amd64'))
+           0:   48 c7 c0 17 00 00 00    mov    rax, 0x17
+        >>> print(disasm(unhex('04001fe552009000'), arch = 'arm'))
            0:   e51f0004        ldr     r0, [pc, #-4]   ; 0x4
            4:   00900052        addseq  r0, r0, r2, asr r0
-        >>> print disasm('4ff00500'.decode('hex'), arch = 'thumb', bits=32)
+        >>> print(disasm(unhex('4ff00500'), arch = 'thumb', bits=32))
            0:   f04f 0005       mov.w   r0, #5
+        >>> print(disasm(unhex('656664676665400F18A4000000000051'), byte=0, arch='amd64'))
+           0:   gs data16 fs data16 rex nop/reserved BYTE PTR gs:[eax+eax*1+0x0]
+           f:   push   rcx
     """
     result = ''
 
     arch   = context.arch
-    os     = context.os
 
     tmpdir = tempfile.mkdtemp(prefix = 'pwn-disasm-')
     step1  = path.join(tmpdir, 'step1')
@@ -763,6 +760,9 @@ def disasm(data, vma = 0, byte = True, offset = True, instructions = True):
         '--rename-section', '.data=.text',
     ]
 
+    if not byte:
+        objdump += ['--no-show-raw-insn']
+
     if arch == 'thumb':
         objcopy += ['--prefix-symbol=$t.']
     else:
@@ -770,10 +770,10 @@ def disasm(data, vma = 0, byte = True, offset = True, instructions = True):
 
     try:
 
-        with open(step1, 'w') as fd:
+        with open(step1, 'wb') as fd:
             fd.write(data)
 
-        res = _run(objcopy + [step1, step2])
+        _run(objcopy + [step1, step2])
 
         output0 = _run(objdump + [step2])
         output1 = output0.split('<.text>:\n')
@@ -789,14 +789,22 @@ def disasm(data, vma = 0, byte = True, offset = True, instructions = True):
 
 
     lines = []
-    pattern = '^( *[0-9a-f]+: *)((?:[0-9a-f]+ )+ *)(.*)'
+    pattern = '^( *[0-9a-f]+: *)', '((?:[0-9a-f]+ )+ *)', '(.*)'
+    if not byte:
+        pattern = pattern[::2]
+    pattern = ''.join(pattern)
     for line in result.splitlines():
-        try:
-            o, b, i = re.search(pattern, line).groups()
-        except:
+        match = re.search(pattern, line)
+        if not match:
             lines.append(line)
             continue
 
+        groups = match.groups()
+        if byte:
+            o, b, i = groups
+        else:
+            o, i = groups
+
         line = ''
 
         if offset:
@@ -807,4 +815,4 @@ def disasm(data, vma = 0, byte = True, offset = True, instructions = True):
             line += i
         lines.append(line)
 
-    return '\n'.join(lines)
+    return re.sub(',([^ ])', r', \1', '\n'.join(lines))
