@@ -1,11 +1,13 @@
 #!/usr/bin/env python2
 from __future__ import absolute_import
+from __future__ import division
 
 import argparse
 import sys
 from string import whitespace
 
 from pwnlib.commandline import common
+from pwnlib.util.fiddling import unhex
 
 parser = common.parser_commands.add_parser(
     'unhex',
@@ -18,13 +20,14 @@ parser.add_argument('hex', nargs='*',
 
 def main(args):
     try:
+        o = getattr(sys.stdout, 'buffer', sys.stdout)
         if not args.hex:
-            s = sys.stdin.read().translate(None, whitespace)
-            sys.stdout.write(s.decode('hex'))
+            s = getattr(sys.stdin, 'buffer', sys.stdin).read().translate(None, whitespace.encode('ascii'))
+            o.write(unhex(s))
         else:
-            sys.stdout.write(''.join(args.hex).decode('hex'))
-    except TypeError, e:
+            o.write(unhex(''.join(args.hex)))
+    except TypeError as e:
         sys.stderr.write(str(e) + '\n')
 
 if __name__ == '__main__':
-    pwnlib.commandline.common.main(__file__)
+    common.main(__file__)
