@@ -919,51 +919,51 @@ F_LOCK = Constant('F_LOCK',1)
 F_TLOCK = Constant('F_TLOCK',2)
 F_TEST = Constant('F_TEST',3)
 STAT64_HAS_BROKEN_ST_INO = Constant('STAT64_HAS_BROKEN_ST_INO',1)
-S_IFMT = Constant('S_IFMT',00170000)
-S_IFSOCK = Constant('S_IFSOCK',0140000)
-S_IFLNK = Constant('S_IFLNK',0120000)
-S_IFREG = Constant('S_IFREG',0100000)
-S_IFBLK = Constant('S_IFBLK',0060000)
-S_IFDIR = Constant('S_IFDIR',0040000)
-S_IFCHR = Constant('S_IFCHR',0020000)
-S_IFIFO = Constant('S_IFIFO',0010000)
-S_ISUID = Constant('S_ISUID',0004000)
-S_ISGID = Constant('S_ISGID',0002000)
-S_ISVTX = Constant('S_ISVTX',0001000)
-S_IRWXU = Constant('S_IRWXU',00700)
-S_IRUSR = Constant('S_IRUSR',00400)
-S_IWUSR = Constant('S_IWUSR',00200)
-S_IXUSR = Constant('S_IXUSR',00100)
-S_IRWXG = Constant('S_IRWXG',00070)
-S_IRGRP = Constant('S_IRGRP',00040)
-S_IWGRP = Constant('S_IWGRP',00020)
-S_IXGRP = Constant('S_IXGRP',00010)
-S_IRWXO = Constant('S_IRWXO',00007)
-S_IROTH = Constant('S_IROTH',00004)
-S_IWOTH = Constant('S_IWOTH',00002)
-S_IXOTH = Constant('S_IXOTH',00001)
-S_IREAD = Constant('S_IREAD',00400)
-S_IWRITE = Constant('S_IWRITE',00200)
-S_IEXEC = Constant('S_IEXEC',00100)
+S_IFMT = Constant('S_IFMT',0o0170000)
+S_IFSOCK = Constant('S_IFSOCK',0o140000)
+S_IFLNK = Constant('S_IFLNK',0o120000)
+S_IFREG = Constant('S_IFREG',0o100000)
+S_IFBLK = Constant('S_IFBLK',0o060000)
+S_IFDIR = Constant('S_IFDIR',0o040000)
+S_IFCHR = Constant('S_IFCHR',0o020000)
+S_IFIFO = Constant('S_IFIFO',0o010000)
+S_ISUID = Constant('S_ISUID',0o004000)
+S_ISGID = Constant('S_ISGID',0o002000)
+S_ISVTX = Constant('S_ISVTX',0o001000)
+S_IRWXU = Constant('S_IRWXU',0o0700)
+S_IRUSR = Constant('S_IRUSR',0o0400)
+S_IWUSR = Constant('S_IWUSR',0o0200)
+S_IXUSR = Constant('S_IXUSR',0o0100)
+S_IRWXG = Constant('S_IRWXG',0o0070)
+S_IRGRP = Constant('S_IRGRP',0o0040)
+S_IWGRP = Constant('S_IWGRP',0o0020)
+S_IXGRP = Constant('S_IXGRP',0o0010)
+S_IRWXO = Constant('S_IRWXO',0o0007)
+S_IROTH = Constant('S_IROTH',0o0004)
+S_IWOTH = Constant('S_IWOTH',0o0002)
+S_IXOTH = Constant('S_IXOTH',0o0001)
+S_IREAD = Constant('S_IREAD',0o0400)
+S_IWRITE = Constant('S_IWRITE',0o0200)
+S_IEXEC = Constant('S_IEXEC',0o0100)
 F_LINUX_SPECIFIC_BASE = Constant('F_LINUX_SPECIFIC_BASE',1024)
-O_ACCMODE = Constant('O_ACCMODE',0003)
-O_RDONLY = Constant('O_RDONLY',00)
-O_WRONLY = Constant('O_WRONLY',01)
-O_RDWR = Constant('O_RDWR',02)
-O_CREAT = Constant('O_CREAT',0100)
-O_EXCL = Constant('O_EXCL',0200)
-O_NOCTTY = Constant('O_NOCTTY',0400)
-O_TRUNC = Constant('O_TRUNC',01000)
-O_APPEND = Constant('O_APPEND',02000)
-O_NONBLOCK = Constant('O_NONBLOCK',04000)
-O_NDELAY = Constant('O_NDELAY',04000)
-O_SYNC = Constant('O_SYNC',010000)
-FASYNC = Constant('FASYNC',020000)
-O_DIRECTORY = Constant('O_DIRECTORY',040000)
-O_NOFOLLOW = Constant('O_NOFOLLOW',0100000)
-O_DIRECT = Constant('O_DIRECT',0200000)
-O_LARGEFILE = Constant('O_LARGEFILE',0400000)
-O_NOATIME = Constant('O_NOATIME',01000000)
+O_ACCMODE = Constant('O_ACCMODE',0o003)
+O_RDONLY = Constant('O_RDONLY',0o0)
+O_WRONLY = Constant('O_WRONLY',0o1)
+O_RDWR = Constant('O_RDWR',0o2)
+O_CREAT = Constant('O_CREAT',0o100)
+O_EXCL = Constant('O_EXCL',0o200)
+O_NOCTTY = Constant('O_NOCTTY',0o400)
+O_TRUNC = Constant('O_TRUNC',0o1000)
+O_APPEND = Constant('O_APPEND',0o2000)
+O_NONBLOCK = Constant('O_NONBLOCK',0o4000)
+O_NDELAY = Constant('O_NDELAY',0o4000)
+O_SYNC = Constant('O_SYNC',0o10000)
+FASYNC = Constant('FASYNC',0o20000)
+O_DIRECTORY = Constant('O_DIRECTORY',0o40000)
+O_NOFOLLOW = Constant('O_NOFOLLOW',0o100000)
+O_DIRECT = Constant('O_DIRECT',0o200000)
+O_LARGEFILE = Constant('O_LARGEFILE',0o400000)
+O_NOATIME = Constant('O_NOATIME',0o1000000)
 F_DUPFD = Constant('F_DUPFD',0)
 F_GETFD = Constant('F_GETFD',1)
 F_SETFD = Constant('F_SETFD',2)
@@ -994,7 +994,7 @@ LOCK_MAND = Constant('LOCK_MAND',32)
 LOCK_READ = Constant('LOCK_READ',64)
 LOCK_WRITE = Constant('LOCK_WRITE',128)
 LOCK_RW = Constant('LOCK_RW',192)
-O_ASYNC = Constant('O_ASYNC',020000)
+O_ASYNC = Constant('O_ASYNC',0o20000)
 MREMAP_MAYMOVE = Constant('MREMAP_MAYMOVE',1)
 MREMAP_FIXED = Constant('MREMAP_FIXED',2)
 PROT_READ = Constant('PROT_READ',0x1)
