@@ -576,32 +576,32 @@ F_ULOCK = Constant('F_ULOCK',0)
 F_LOCK = Constant('F_LOCK',1)
 F_TLOCK = Constant('F_TLOCK',2)
 F_TEST = Constant('F_TEST',3)
-S_IFMT = Constant('S_IFMT',00170000)
-S_IFSOCK = Constant('S_IFSOCK',0140000)
-S_IFLNK = Constant('S_IFLNK',0120000)
-S_IFREG = Constant('S_IFREG',0100000)
-S_IFBLK = Constant('S_IFBLK',0060000)
-S_IFDIR = Constant('S_IFDIR',0040000)
-S_IFCHR = Constant('S_IFCHR',0020000)
-S_IFIFO = Constant('S_IFIFO',0010000)
-S_ISUID = Constant('S_ISUID',0004000)
-S_ISGID = Constant('S_ISGID',0002000)
-S_ISVTX = Constant('S_ISVTX',0001000)
-S_IRWXU = Constant('S_IRWXU',00700)
-S_IRUSR = Constant('S_IRUSR',00400)
-S_IWUSR = Constant('S_IWUSR',00200)
-S_IXUSR = Constant('S_IXUSR',00100)
-S_IRWXG = Constant('S_IRWXG',00070)
-S_IRGRP = Constant('S_IRGRP',00040)
-S_IWGRP = Constant('S_IWGRP',00020)
-S_IXGRP = Constant('S_IXGRP',00010)
-S_IRWXO = Constant('S_IRWXO',00007)
-S_IROTH = Constant('S_IROTH',00004)
-S_IWOTH = Constant('S_IWOTH',00002)
-S_IXOTH = Constant('S_IXOTH',00001)
-S_IREAD = Constant('S_IREAD',00400)
-S_IWRITE = Constant('S_IWRITE',00200)
-S_IEXEC = Constant('S_IEXEC',00100)
+S_IFMT = Constant('S_IFMT',0o0170000)
+S_IFSOCK = Constant('S_IFSOCK',0o140000)
+S_IFLNK = Constant('S_IFLNK',0o120000)
+S_IFREG = Constant('S_IFREG',0o100000)
+S_IFBLK = Constant('S_IFBLK',0o060000)
+S_IFDIR = Constant('S_IFDIR',0o040000)
+S_IFCHR = Constant('S_IFCHR',0o020000)
+S_IFIFO = Constant('S_IFIFO',0o010000)
+S_ISUID = Constant('S_ISUID',0o004000)
+S_ISGID = Constant('S_ISGID',0o002000)
+S_ISVTX = Constant('S_ISVTX',0o001000)
+S_IRWXU = Constant('S_IRWXU',0o0700)
+S_IRUSR = Constant('S_IRUSR',0o0400)
+S_IWUSR = Constant('S_IWUSR',0o0200)
+S_IXUSR = Constant('S_IXUSR',0o0100)
+S_IRWXG = Constant('S_IRWXG',0o0070)
+S_IRGRP = Constant('S_IRGRP',0o0040)
+S_IWGRP = Constant('S_IWGRP',0o0020)
+S_IXGRP = Constant('S_IXGRP',0o0010)
+S_IRWXO = Constant('S_IRWXO',0o0007)
+S_IROTH = Constant('S_IROTH',0o0004)
+S_IWOTH = Constant('S_IWOTH',0o0002)
+S_IXOTH = Constant('S_IXOTH',0o0001)
+S_IREAD = Constant('S_IREAD',0o0400)
+S_IWRITE = Constant('S_IWRITE',0o0200)
+S_IEXEC = Constant('S_IEXEC',0o0100)
 F_LINUX_SPECIFIC_BASE = Constant('F_LINUX_SPECIFIC_BASE',1024)
 O_RDONLY = Constant('O_RDONLY',0x0000)
 O_WRONLY = Constant('O_WRONLY',0x0001)
