@@ -10,7 +10,7 @@ have been useful.
 
     >>> context.clear(arch='i386')
     >>> binary = ELF.from_assembly('add esp, 0x10; ret')
-    >>> binary.symbols = {'read': 0xdeadbeef, 'write': 0xdecafbad, 'exit': 0xfeedface}
+    >>> binary.symbols = {'read': 0xdeadbeef, 'write': 0xdecafbad, 'execve': 0xcafebabe, 'exit': 0xfeedface}
 
 Creating a ROP object which looks up symbols in the binary is
 pretty straightforward.
@@ -20,13 +20,13 @@ pretty straightforward.
 With the ROP object, you can manually add stack frames.
 
     >>> rop.raw(0)
-    >>> rop.raw(unpack('abcd'))
+    >>> rop.raw(unpack(b'abcd'))
     >>> rop.raw(2)
 
 Inspecting the ROP stack is easy, and laid out in an easy-to-read
 manner.
 
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:              0x0
     0x0004:       0x64636261
     0x0008:              0x2
@@ -35,12 +35,12 @@ The ROP module is also aware of how to make function calls with
 standard Linux ABIs.
 
     >>> rop.call('read', [4,5,6])
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:              0x0
     0x0004:       0x64636261
     0x0008:              0x2
     0x000c:       0xdeadbeef read(4, 5, 6)
-    0x0010:           'eaaa' <return address>
+    0x0010:          b'eaaa' <return address>
     0x0014:              0x4 arg0
     0x0018:              0x5 arg1
     0x001c:              0x6 arg2
@@ -50,7 +50,7 @@ The stack is automatically adjusted for the next frame
 
     >>> rop.write(7,8,9)
     >>> rop.exit()
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:              0x0
     0x0004:       0x64636261
     0x0008:              0x2
@@ -59,15 +59,36 @@ The stack is automatically adjusted for the next frame
     0x0014:              0x4 arg0
     0x0018:              0x5 arg1
     0x001c:              0x6 arg2
-    0x0020:           'iaaa' <pad>
+    0x0020:          b'iaaa' <pad>
     0x0024:       0xdecafbad write(7, 8, 9)
     0x0028:       0x10000000 <adjust @0x3c> add esp, 0x10; ret
     0x002c:              0x7 arg0
     0x0030:              0x8 arg1
     0x0034:              0x9 arg2
-    0x0038:           'oaaa' <pad>
+    0x0038:          b'oaaa' <pad>
     0x003c:       0xfeedface exit()
 
+You can also append complex arguments onto stack when the stack pointer is known.
+
+    >>> rop = ROP(binary, base=0x7fffe000)
+    >>> rop.call('execve', ['/bin/sh', [['/bin/sh'], ['-p'], ['-c'], ['ls']], 0])
+    >>> print(rop.dump())
+    0x7fffe000:       0xcafebabe execve(['/bin/sh'], [['/bin/sh'], ['-p'], ['-c'], ['ls']], 0)
+    0x7fffe004:          b'baaa' <return address>
+    0x7fffe008:       0x7fffe014 arg0 (+0xc)
+    0x7fffe00c:       0x7fffe01c arg1 (+0x10)
+    0x7fffe010:              0x0 arg2
+    0x7fffe014:   b'/bin/sh\x00'
+    0x7fffe01c:       0x7fffe02c (+0x10)
+    0x7fffe020:       0x7fffe034 (+0x14)
+    0x7fffe024:       0x7fffe038 (+0x14)
+    0x7fffe028:       0x7fffe03c (+0x14)
+    0x7fffe02c:   b'/bin/sh\x00'
+    0x7fffe034:       b'-p\x00$'
+    0x7fffe038:       b'-c\x00$'
+    0x7fffe03c:       b'ls\x00$'
+
+
 ROP Example
 -------------------
 
@@ -104,27 +125,27 @@ Finally, let's build our ROP stack
     >>> rop = ROP(binary)
     >>> rop.write(c.STDOUT_FILENO, binary.symbols['flag'], 8)
     >>> rop.exit()
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:       0x10000012 write(STDOUT_FILENO, 0x10000026, 8)
     0x0004:       0x1000000e <adjust @0x18> add esp, 0x10; ret
-    0x0008:              0x1 arg0
+    0x0008:              0x1 STDOUT_FILENO
     0x000c:       0x10000026 flag
     0x0010:              0x8 arg2
-    0x0014:           'faaa' <pad>
+    0x0014:          b'faaa' <pad>
     0x0018:       0x1000002f exit()
 
 The raw data from the ROP stack is available via `str`.
 
-    >>> raw_rop = str(rop)
-    >>> print enhex(raw_rop)
+    >>> raw_rop = rop.chain()
+    >>> print(enhex(raw_rop))
     120000100e000010010000002600001008000000666161612f000010
 
 Let's try it out!
 
     >>> p = process(binary.path)
     >>> p.send(raw_rop)
-    >>> print p.recvall(timeout=5)
-    The flag
+    >>> print(repr(p.recvall(timeout=5)))
+    b'The flag'
 
 ROP Example (amd64)
 -------------------
@@ -138,34 +159,34 @@ requirements so that everything "just works".
     >>> binary = ELF.from_assembly(assembly)
     >>> rop = ROP(binary)
     >>> rop.target(1,2,3)
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:       0x10000000 pop rdx; pop rdi; pop rsi; add rsp, 0x20; ret
     0x0008:              0x3 [arg2] rdx = 3
     0x0010:              0x1 [arg0] rdi = 1
     0x0018:              0x2 [arg1] rsi = 2
-    0x0020:       'iaaajaaa' <pad 0x20>
-    0x0028:       'kaaalaaa' <pad 0x18>
-    0x0030:       'maaanaaa' <pad 0x10>
-    0x0038:       'oaaapaaa' <pad 0x8>
+    0x0020:      b'iaaajaaa' <pad 0x20>
+    0x0028:      b'kaaalaaa' <pad 0x18>
+    0x0030:      b'maaanaaa' <pad 0x10>
+    0x0038:      b'oaaapaaa' <pad 0x8>
     0x0040:       0x10000008 target
     >>> rop.target(1)
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:       0x10000000 pop rdx; pop rdi; pop rsi; add rsp, 0x20; ret
     0x0008:              0x3 [arg2] rdx = 3
     0x0010:              0x1 [arg0] rdi = 1
     0x0018:              0x2 [arg1] rsi = 2
-    0x0020:       'iaaajaaa' <pad 0x20>
-    0x0028:       'kaaalaaa' <pad 0x18>
-    0x0030:       'maaanaaa' <pad 0x10>
-    0x0038:       'oaaapaaa' <pad 0x8>
+    0x0020:      b'iaaajaaa' <pad 0x20>
+    0x0028:      b'kaaalaaa' <pad 0x18>
+    0x0030:      b'maaanaaa' <pad 0x10>
+    0x0038:      b'oaaapaaa' <pad 0x8>
     0x0040:       0x10000008 target
     0x0048:       0x10000001 pop rdi; pop rsi; add rsp, 0x20; ret
     0x0050:              0x1 [arg0] rdi = 1
-    0x0058:       'waaaxaaa' <pad rsi>
-    0x0060:       'yaaazaab' <pad 0x20>
-    0x0068:       'baabcaab' <pad 0x18>
-    0x0070:       'daabeaab' <pad 0x10>
-    0x0078:       'faabgaab' <pad 0x8>
+    0x0058:      b'waaaxaaa' <pad rsi>
+    0x0060:      b'yaaazaab' <pad 0x20>
+    0x0068:      b'baabcaab' <pad 0x18>
+    0x0070:      b'daabeaab' <pad 0x10>
+    0x0078:      b'faabgaab' <pad 0x8>
     0x0080:       0x10000008 target
 
 Pwntools will also filter out some bad instructions while setting the registers
@@ -175,12 +196,12 @@ Pwntools will also filter out some bad instructions while setting the registers
     >>> binary = ELF.from_assembly(assembly)
     >>> rop = ROP(binary)
     >>> rop.call(0xdeadbeef, [1, 2, 3])
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:       0x1000000b pop rdi; ret
     0x0008:              0x1 [arg0] rdi = 1
-    0x0010:       0x10000008 pop rsi; pop rdx; ret
-    0x0018:              0x2 [arg1] rsi = 2
-    0x0020:              0x3 [arg2] rdx = 3
+    0x0010:       0x10000002 pop rdx; pop rsi; ret
+    0x0018:              0x3 [arg2] rdx = 3
+    0x0020:              0x2 [arg1] rsi = 2
     0x0028:       0xdeadbeef
 
 ROP + Sigreturn
@@ -213,7 +234,7 @@ Let's create a ROP object and invoke the call.
 
 That's all there is to it.
 
-    >>> print rop.dump()
+    >>> print(rop.dump())
     0x0000:       0x1000000e pop eax; ret
     0x0004:             0x77 [arg0] eax = SYS_sigreturn
     0x0008:       0x1000000b int 0x80
@@ -228,7 +249,7 @@ That's all there is to it.
     0x002c:       0x10000012 ebx = binsh
     0x0030:              0x0 edx
     0x0034:              0x0 ecx
-    0x0038:              0xb eax
+    0x0038:              0xb eax = SYS_execve
     0x003c:              0x0 trapno
     0x0040:              0x0 err
     0x0044:       0x1000000b int 0x80
@@ -241,13 +262,14 @@ That's all there is to it.
 Let's try it out!
 
     >>> p = process(binary.path)
-    >>> p.send(str(rop))
+    >>> p.send(rop.chain())
     >>> time.sleep(1)
-    >>> p.sendline('echo hello; exit')
+    >>> p.sendline(b'echo hello; exit')
     >>> p.recvline()
-    'hello\n'
+    b'hello\n'
 """
 from __future__ import absolute_import
+from __future__ import division
 
 import collections
 import copy
@@ -255,6 +277,8 @@ import hashlib
 import itertools
 import os
 import re
+import shutil
+import six
 import string
 import sys
 import tempfile
@@ -266,20 +290,29 @@ from pwnlib.context import context
 from pwnlib.elf import ELF
 from pwnlib.log import getLogger
 from pwnlib.rop import srop
+from . import ret2dlresolve
 from pwnlib.rop.call import AppendedArgument
 from pwnlib.rop.call import Call
 from pwnlib.rop.call import CurrentStackPointer
 from pwnlib.rop.call import NextGadgetAddress
 from pwnlib.rop.call import StackAdjustment
+from pwnlib.rop.call import Unresolved
 from pwnlib.rop.gadgets import Gadget
 from pwnlib.util import lists
 from pwnlib.util import packing
 from pwnlib.util.cyclic import cyclic
-from pwnlib.util.packing import *
+from pwnlib.util.packing import pack
 
 log = getLogger(__name__)
 __all__ = ['ROP']
 
+enums = Call, constants.Constant
+try:
+    from enum import Enum
+except ImportError:
+    pass
+else:
+    enums += Enum,
 
 class Padding(object):
     """
@@ -288,25 +321,36 @@ class Padding(object):
     def __init__(self, name='<pad>'):
         self.name = name
 
+def _slot_len(x):
+    if isinstance(x, six.integer_types+(Unresolved, Padding, Gadget)):
+        return context.bytes
+    else:
+        return len(packing.flat(x))
+
 class DescriptiveStack(list):
     """
     List of resolved ROP gadgets that correspond to the ROP calls that
-    the user has specified.  Also includes
+    the user has specified.
     """
 
     #: Base address
     address = 0
 
-    #: Dictionary of ``{address: [list of descriptions]}``
+    #: Dictionary of \`{address: [list of descriptions]}`
     descriptions = {}
 
     def __init__(self, address):
         self.descriptions = collections.defaultdict(lambda: [])
         self.address      = address or 0
+        self._next_next   = 0
+        self._next_last   = 0
 
     @property
     def next(self):
-        return self.address + len(self) * context.bytes
+        for x in self[self._next_last:]:
+            self._next_next += _slot_len(x)
+        self._next_last = len(self)
+        return self.address + self._next_next
 
     def describe(self, text, address = None):
         if address is None:
@@ -315,13 +359,13 @@ class DescriptiveStack(list):
 
     def dump(self):
         rv = []
+        addr = self.address
         for i, data in enumerate(self):
-            addr = self.address + i * context.bytes
             off = None
             line = '0x%04x:' % addr
-            if isinstance(data, str):
+            if isinstance(data, (str, bytes)):
                 line += ' %16r' % data
-            elif isinstance(data, (int,long)):
+            elif isinstance(data, six.integer_types):
                 line += ' %#16x' % data
                 if self.address != 0 and self.address < data < self.next:
                     off = data - addr
@@ -333,6 +377,7 @@ class DescriptiveStack(list):
             if off is not None:
                 line += ' (+%#x)' % off
             rv.append(line)
+            addr += _slot_len(data)
 
         return '\n'.join(rv)
 
@@ -352,7 +397,7 @@ class ROP(object):
        #  '0x0004:       0xdeadbeef',
        #  '0x0008:              0x0',
        #  '0x000c:        0x80496a8']
-       str(rop)
+       bytes(rop)
        # '\xfc\x82\x04\x08\xef\xbe\xad\xde\x00\x00\x00\x00\xa8\x96\x04\x08'
 
     >>> context.clear(arch = "i386", kernel = 'amd64')
@@ -363,13 +408,13 @@ class ROP(object):
     >>> r.funcname(1, 2)
     >>> r.funcname(3)
     >>> r.execve(4, 5, 6)
-    >>> print r.dump()
+    >>> print(r.dump())
     0x0000:       0x10001234 funcname(1, 2)
     0x0004:       0x10000003 <adjust @0x18> add esp, 0x10; ret
     0x0008:              0x1 arg0
     0x000c:              0x2 arg1
-    0x0010:           'eaaa' <pad>
-    0x0014:           'faaa' <pad>
+    0x0010:          b'eaaa' <pad>
+    0x0014:          b'faaa' <pad>
     0x0018:       0x10001234 funcname(3)
     0x001c:       0x10000007 <adjust @0x24> pop eax; ret
     0x0020:              0x3 arg0
@@ -387,7 +432,7 @@ class ROP(object):
     0x0050:              0x4 ebx
     0x0054:              0x6 edx
     0x0058:              0x5 ecx
-    0x005c:              0xb eax
+    0x005c:              0xb eax = SYS_execve
     0x0060:              0x0 trapno
     0x0064:              0x0 err
     0x0068:       0x10000000 int 0x80
@@ -401,13 +446,13 @@ class ROP(object):
     >>> r.funcname(1, 2)
     >>> r.funcname(3)
     >>> r.execve(4, 5, 6)
-    >>> print r.dump()
+    >>> print(r.dump())
     0x8048000:       0x10001234 funcname(1, 2)
     0x8048004:       0x10000003 <adjust @0x8048018> add esp, 0x10; ret
     0x8048008:              0x1 arg0
     0x804800c:              0x2 arg1
-    0x8048010:           'eaaa' <pad>
-    0x8048014:           'faaa' <pad>
+    0x8048010:          b'eaaa' <pad>
+    0x8048014:          b'faaa' <pad>
     0x8048018:       0x10001234 funcname(3)
     0x804801c:       0x10000007 <adjust @0x8048024> pop eax; ret
     0x8048020:              0x3 arg0
@@ -425,7 +470,7 @@ class ROP(object):
     0x8048050:              0x4 ebx
     0x8048054:              0x6 edx
     0x8048058:              0x5 ecx
-    0x804805c:              0xb eax
+    0x804805c:              0xb eax = SYS_execve
     0x8048060:              0x0 trapno
     0x8048064:              0x0 err
     0x8048068:       0x10000000 int 0x80
@@ -440,13 +485,13 @@ class ROP(object):
     >>> r = ROP(elf)
     >>> r.ret.address == 0x10000000
     True
-    >>> r = ROP(elf, badchars='\x00')
+    >>> r = ROP(elf, badchars=b'\x00')
     >>> r.gadgets == {}
     True
     >>> r.ret is None
     True
     """
-    def __init__(self, elfs, base = None, badchars = '', **kwargs):
+    def __init__(self, elfs, base = None, badchars = b'', **kwargs):
         """
         Arguments:
             elfs(list): List of :class:`.ELF` objects for mining
@@ -458,7 +503,7 @@ class ROP(object):
         # Permit singular ROP(elf) vs ROP([elf])
         if isinstance(elfs, ELF):
             elfs = [elfs]
-        elif isinstance(elfs, (str, unicode)):
+        elif isinstance(elfs, (bytes, six.text_type)):
             elfs = [ELF(elfs)]
 
         #: List of individual ROP gadgets, ROP calls, SROP frames, etc.
@@ -544,7 +589,7 @@ class ROP(object):
         budget = 999999999
 
         for num_gadgets in range(len(registers)):
-            for combo in itertools.combinations(best_gadgets.values(), 1+num_gadgets):
+            for combo in itertools.combinations(sorted(best_gadgets.values(), key=repr, reverse=True), 1+num_gadgets):
                 # Is this better than what we can already do?
                 cost = sum((g.move for g in combo))
                 if cost > budget:
@@ -598,7 +643,7 @@ class ROP(object):
                 if resolvable in elf.symbols:
                     return elf.symbols[resolvable]
 
-        if isinstance(resolvable, (int, long)):
+        if isinstance(resolvable, six.integer_types):
             return resolvable
 
     def unresolve(self, value):
@@ -633,27 +678,27 @@ class ROP(object):
         >>> len(val)
         15
         >>> rop.generatePadding(0,0)
-        ''
+        b''
 
         """
 
         # Ensure we don't generate a cyclic pattern which contains badchars
-        alphabet = ''.join(c for c in string.ascii_lowercase if c not in self._badchars)
+        alphabet = b''.join(packing.p8(c) for c in bytearray(string.ascii_lowercase.encode()) if c not in self._badchars)
 
         if count:
             return cyclic(offset + count, alphabet=alphabet)[-count:]
-        return ''
+        return b''
 
     def describe(self, object):
         """
         Return a description for an object in the ROP stack
         """
-        if isinstance(object, (int, long)):
-            return self.unresolve(object)
-        if isinstance(object, str):
-            return repr(object)
-        if isinstance(object, Call):
+        if isinstance(object, enums):
             return str(object)
+        if isinstance(object, six.integer_types):
+            return self.unresolve(object)
+        if isinstance(object, (bytes, six.text_type)):
+            return repr(object)
         if isinstance(object, Gadget):
             return '; '.join(object.insns)
 
@@ -695,15 +740,17 @@ class ROP(object):
 
             # Integers can just be added.
             # Do our best to find out what the address is.
-            if isinstance(slot, (int, long)):
+            if isinstance(slot, six.integer_types):
                 stack.describe(self.describe(slot))
                 stack.append(slot)
 
 
             # Byte blobs can also be added, however they must be
             # broken down into pointer-width blobs.
-            elif isinstance(slot, (str, unicode)):
+            elif isinstance(slot, (bytes, six.text_type)):
                 stack.describe(self.describe(slot))
+                if not isinstance(slot, bytes):
+                    slot = slot.encode()
                 slot += self.generatePadding(stack.next, len(slot) % context.bytes)
 
                 for chunk in lists.group(context.bytes, slot):
@@ -728,13 +775,13 @@ class ROP(object):
             elif isinstance(slot, Call):
                 stack.describe(self.describe(slot))
 
-                registers    = dict(zip(slot.abi.register_arguments, slot.args))
+                registers    = slot.register_arguments
 
                 for value, name in self.setRegisters(registers):
                     if name in registers:
                         index = slot.abi.register_arguments.index(name)
-                        description = self.describe(value) or value
-                        stack.describe('[arg%s] %s = %s' % (index, name, description))
+                        description = self.describe(value) or repr(value)
+                        stack.describe('[arg%d] %s = %s' % (index, name, description))
                     elif isinstance(name, Gadget):
                         stack.describe('; '.join(name.insns))
                     elif isinstance(name, str):
@@ -747,7 +794,10 @@ class ROP(object):
                 stack.append(slot.target)
 
                 # For any remaining arguments, put them on the stack
-                stackArguments = slot.args[len(slot.abi.register_arguments):]
+                stackArguments = slot.stack_arguments
+                for argument in slot.stack_arguments_before:
+                    stack.describe("[dlresolve index]")
+                    stack.append(argument)
                 nextGadgetAddr = stack.next + (context.bytes * len(stackArguments))
 
                 # Generally, stack-based arguments assume there's a return
@@ -807,17 +857,17 @@ class ROP(object):
         start = base
         end   = stack.next
         size  = (stack.next - base)
+        slot_address = base
         for i, slot in enumerate(stack):
-            slot_address = stack.address + (i * context.bytes)
-            if isinstance(slot, (int, long)):
+            if isinstance(slot, six.integer_types):
                 pass
 
-            elif isinstance(slot, (str, unicode)):
+            elif isinstance(slot, (bytes, six.text_type)):
                 pass
 
             elif isinstance(slot, AppendedArgument):
                 stack[i] = stack.next
-                stack.extend(slot.resolve(stack.next))
+                stack.extend(slot.resolve(stack.next + len(slot) - context.bytes))
 
             elif isinstance(slot, CurrentStackPointer):
                 stack[i] = slot_address
@@ -836,11 +886,13 @@ class ROP(object):
             else:
                 pass
 
+            slot_address += _slot_len(slot)
+
         return stack
 
 
     def find_stack_adjustment(self, slots):
-        self.search(move=slots * context.arch)
+        self.search(move=slots * context.bytes)
 
     def chain(self):
         """Build the ROP chain
@@ -925,7 +977,10 @@ class ROP(object):
                 SYS_sigreturn  = constants.SYS_rt_sigreturn
 
             for register, value in zip(frame.arguments, arguments):
-                frame[register] = value
+                if not isinstance(value, six.integer_types + (Unresolved,)):
+                    frame[register] = AppendedArgument(value)
+                else:
+                    frame[register] = value
 
         # Set up a call frame which will set EAX and invoke the syscall
         call = Call('SYS_sigreturn',
@@ -968,13 +1023,13 @@ class ROP(object):
         >>> rop.raw('AAAAAAAA')
         >>> rop.raw('BBBBBBBB')
         >>> rop.raw('CCCCCCCC')
-        >>> print rop.dump()
-        0x0000:           'AAAA' 'AAAAAAAA'
-        0x0004:           'AAAA'
-        0x0008:           'BBBB' 'BBBBBBBB'
-        0x000c:           'BBBB'
-        0x0010:           'CCCC' 'CCCCCCCC'
-        0x0014:           'CCCC'
+        >>> print(rop.dump())
+        0x0000:          b'AAAA' 'AAAAAAAA'
+        0x0004:          b'AAAA'
+        0x0008:          b'BBBB' 'BBBBBBBB'
+        0x000c:          b'BBBB'
+        0x0010:          b'CCCC' 'CCCCCCCC'
+        0x0014:          b'CCCC'
         """
         if self.migrated:
             log.error('Cannot append to a migrated chain')
@@ -998,39 +1053,46 @@ class ROP(object):
             log.error('Cannot find the gadgets to migrate')
         self.migrated = True
 
-    def __str__(self):
+    def __bytes__(self):
         """Returns: Raw bytes of the ROP chain"""
         return self.chain()
 
+    def __str__(self):
+        return str(self.chain())
+
     def __get_cachefile_name(self, files):
         """Given an ELF or list of ELF objects, return a cache file for the set of files"""
-        cachedir = os.path.join(tempfile.gettempdir(), 'pwntools-rop-cache')
+        cachedir = os.path.join(tempfile.gettempdir(), 'pwntools-rop-cache-%d.%d' % sys.version_info[:2])
         if not os.path.exists(cachedir):
             os.mkdir(cachedir)
 
         if isinstance(files, ELF):
             files = [files]
 
-        hashes = []
+        sha256 = hashlib.sha256()
+        for elf_data in sorted(elf.get_data() for elf in files):
+            sha256.update(elf_data)
 
-        for elf in self.elfs:
-            sha256 = hashlib.sha256(elf.get_data()).hexdigest()
-            hashes.append(sha256)
+        return os.path.join(cachedir, sha256.hexdigest())
 
-        return os.path.join(cachedir, '_'.join(hashes))
+    @staticmethod
+    def clear_cache():
+        """Clears the ROP gadget cache"""
+        cachedir = os.path.join(tempfile.gettempdir(), 'pwntools-rop-cache')
+        shutil.rmtree(cachedir)
 
     def __cache_load(self, elf):
         filename = self.__get_cachefile_name(elf)
         if not os.path.exists(filename):
             return None
-        log.info_once('Loaded cached gadgets for %r' % elf.file.name)
-        gadgets = eval(file(filename).read())
+        gadgets = eval(open(filename).read())
         gadgets = {k - elf.load_addr + elf.address:v for k, v in gadgets.items()}
+        log.info_once('Loaded %s cached gadgets for %r', len(gadgets), elf.file.name)
         return gadgets
 
     def __cache_save(self, elf, data):
         data = {k + elf.load_addr - elf.address:v for k, v in data.items()}
-        file(self.__get_cachefile_name(elf), 'w+').write(repr(data))
+        open(self.__get_cachefile_name(elf), 'w+').write(repr(data))
 
     def __load(self):
         """Load all ROP gadgets for the selected ELF files"""
@@ -1229,6 +1291,20 @@ class ROP(object):
 
         return result
 
+    def ret2dlresolve(self, dlresolve):
+        elf = next(elf for elf in self.elfs if elf.get_section_by_name(".plt"))
+        elf_base = elf.address if elf.pie else 0
+        plt_init = elf.get_section_by_name(".plt").header.sh_addr + elf_base
+        log.debug("PLT_INIT: %#x", plt_init)
+
+        reloc_index = dlresolve.reloc_index
+        real_args = dlresolve.real_args
+        call = Call("[plt_init] " + dlresolve.symbol.decode(),
+                    plt_init,
+                    dlresolve.real_args,
+                    before=[reloc_index])
+        self.raw(call)
+
     def __getattr__(self, attr):
         """Helper to make finding ROP gadets easier.
 
